/* Copyright (C) 2003 Alex de Landgraaf <alextreme@xs4all.nl>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <glib.h>
#include <stdio.h>
#include <stdarg.h>
#include "args.h"
#include "installer.h"

void print_help() {
	printf("Usage:\n morphixinstaller --debug | -d\n\t\tPrint debug messages\n\
 morphixinstaller --root <path_to_rootpartitiondevice>\n\t\tUse the specified rootpartition\n\
 morphixinstaller --swap <part_to_swappartitiondevice>\n\t\tUse the specified swappartition\n\
 morphixinstaller --hdd <3-letter-harddiskdevice>\n\t\tUse the specified harddisk\n\
 morphixinstaller --nopart\n\t\tDon't partition, go straight to rootpartition selector\n\
 morphixinstaller --noswap\n\t\tDon't use swap at all, skip section\n\
 morphixinstaller --part-with <path_to_application>\n\t\tWill execute the specified program instead of the default\n\
 morphixinstaller --fstype <ext2|ext3|reiserfs|xfs>\n\t\tWill skip filesystemselection\n\
 morphixinstaller --hostname <alfanumeric string>\n\t\tUse the specified hostname for installing\n\
 morphixinstaller --rootpass <string>\n\t\tUse the specified string as password for the root user\n\
 morphixinstaller --user <alfanumeric string> <string>\n\t\tUse the specified string as the username for the user being created, and the string after that for the user's password\n\
 morphixinstaller --lilo-mbr\n\t\tUse the selected harddisk for installing lilo on\n\
 morphixinstaller --lilo-root\n\t\tUse the selected rootpartition for installing lilo on\n\
 morphixinstaller --nolilo\n\t\tDon't install lilo\n\
 morphixinstaller --lilo-boot\n\t\tUse the selected bootpartition for installing lilo on\n\
 morphixinstaller --ui [cmdline|gtk|auto]\n\t\tUse the selected installer UI. Auto works only with one harddisk, one Linux partition and one Swap partition. Defaults are ext3 for the fs, Gnoppix for the hostname, morphix for the root password and morph/morph for the username/password. Lilo will be installed to the mbr. You can specify different values by using one of the above arguments. If you don't: MAKE SURE TO CHANGE THE PASSWORDS AFTER INSTALLING! I won't answer any email from people being hacked because they havn't changed their passwords, it is at your own risk!\n\
 \n\
 You may use any combination of these arguments, note that the lilo-options override eachother, the last one counts\n\
 \n\
 morphixinstaller --help | -h\n\t\tPrints commandline arguments and exits\n\
 \n\
 For questions, bugs, feedback or lynching please contact me via www.morphix.org\n\
 This program is licensed under the GNU GPL v2, copyright Alex de Landgraaf, 2003\n\n");
}

gboolean check_args(int *argcp, char ***argvp) {
	gint i;
	gint argc;
	argc = *argcp;
	gchar **argv;
	argv = *argvp;
	cmdlineargs.no_partitioning = FALSE;
	cmdlineargs.no_swap_partition = FALSE;
	cmdlineargs.swappart_set = FALSE;
	cmdlineargs.rootpart_set = FALSE;
	cmdlineargs.harddisk_set = FALSE;
	cmdlineargs.partitioner_set = FALSE;
	cmdlineargs.fs_set = FALSE;
	cmdlineargs.hostname_set = FALSE;
	cmdlineargs.rootpass_set = FALSE;
	cmdlineargs.user_set = FALSE;
	cmdlineargs.lilo_set = FALSE;
	cmdlineargs.ui_set = FALSE;
	cmdlineargs.ui_type = UI_GTK_TYPE;
	if (argc == 1) 
		return TRUE;

	/*
	  a dirty forloop-continue-return construction.
	  couldn't be bothered doing it proper, sorry :)
	*/

	for (i = 1; i < argc; i++) {
		if (strcmp(argv[i],"--debug") == 0 ||
		    strcmp(argv[i],"-d") == 0) {
			debug_switch = 1;
			continue;
		}
		if (strcmp(argv[i],"--help") == 0 ||
		    strcmp(argv[i],"-h") == 0) {
			return FALSE;
		}
		if (strcmp(argv[i],"--root") == 0) {
			if (++i >= argc)
				return FALSE;
			if (strlen(argv[i]) >= 15 ||
			    !g_file_test(argv[i],G_FILE_TEST_EXISTS)) {
				printf("Root argument too long or invalid\n");
				return FALSE;
			}
			strcpy(cmdlineargs.rootpart,argv[i]);
			cmdlineargs.rootpart_set = TRUE;
			continue;
		}
		if (strcmp(argv[i],"--swap") == 0) {
			if (++i >= argc)
				return FALSE;
			if (strlen(argv[i]) >= 15 ||
			    !g_file_test(argv[i],G_FILE_TEST_EXISTS)) {
				printf("Swap argument too long or invalid\n");
				return FALSE;
			}
			strcpy(cmdlineargs.swappart,argv[i]);
			cmdlineargs.swappart_set = TRUE;
			continue;
		}
		if (strcmp(argv[i],"--hdd") == 0) {
			if (++i >= argc)
				return FALSE;
			if (strlen(argv[i]) != 3) {
				printf("Hdd argument too long or invalid, needs to be 3 letters in length\n");
				return FALSE;
			}
			strcpy(cmdlineargs.harddisk,argv[i]);
			cmdlineargs.harddisk_set = TRUE;
			continue;
		}	
		if (strcmp(argv[i],"--nopart") == 0) {
			cmdlineargs.no_partitioning = TRUE;
			continue;
		}
		if (strcmp(argv[i],"--noswap") == 0) {
			cmdlineargs.no_swap_partition = TRUE;
			continue;
		}
		if (strcmp(argv[i],"--part-with") == 0) {
			if (++i >= argc)
				return FALSE;
			if (strlen(argv[i]) >= 63 ||
			    !g_file_test(argv[i],G_FILE_TEST_EXISTS)) {
				printf("Partitioner argument too long or invalid\n");
				return FALSE;
			}
			strcpy(cmdlineargs.partitioner,argv[i]);
			cmdlineargs.partitioner_set = TRUE;
			continue;
		}
		if (strcmp(argv[i],"--fs") == 0) {
			if (++i >= argc) 
				return FALSE;
			if ((strcmp(argv[i],"ext2") == 0) || (strcmp(argv[i],"ext3") == 0) || (strcmp(argv[i],"reiserfs") == 0) || (strcmp(argv[i],"xfs") == 0)) {
				strcpy(cmdlineargs.fs,argv[i]);
			} else {
				printf("fs argument invalid\n");
				return FALSE;
			}
			cmdlineargs.fs_set = TRUE;
			continue;
		}
		if (strcmp(argv[i],"--hostname") == 0) {
			if (++i >= argc)
				return FALSE;
			if (strlen(argv[i]) >= 63) {
				printf("hostname argument too long or invalid\n");
				return FALSE;
			}
			gint j;
			for (j = 0; j < strlen(argv[i]); j++) {
				if (!g_ascii_isalnum(argv[i][j])) {
					printf("hostname argument contains invalid char\n");
					return FALSE;
				}
			}
			strcpy(cmdlineargs.hostname,argv[i]);
			cmdlineargs.hostname_set = TRUE;
			continue;
		}
		if (strcmp(argv[i],"--rootpass") == 0) {
			if (++i >= argc)
				return FALSE;
			if (strlen(argv[i]) >= 63) {
				printf("rootpass argument too long or invalid\n");
				return FALSE;
			}
			gint j;
			for (j = 0; j < strlen(argv[i]); j++) {
				if (!g_ascii_isalnum(argv[i][j])) {
					printf("rootpass argument contains invalid char\n");
					return FALSE;
				}
			}
			strcpy(cmdlineargs.rootpass,argv[i]);
			cmdlineargs.rootpass_set = TRUE;
			continue;
		}
		if (strcmp(argv[i],"--user") == 0) {
			if (++i >= argc) {
				printf("Need username and password\n");
				return FALSE;
			}
			if (strlen(argv[i]) >= 63) {
				printf("username argument too long or invalid\n");
				return FALSE;
			}
			gint j;
			for (j = 0; j < strlen(argv[i]); j++) {
				if (!g_ascii_isalnum(argv[i][j])) {
					printf("username argument contains invalid char\n");
					return FALSE;
				}
			}
			strcpy(cmdlineargs.username,argv[i]);

			if (++i >= argc) {
				printf("Need username and password\n");
				return FALSE;
			}
			if (strlen(argv[i]) >= 63) {
				printf("password argument too long or invalid\n");
				return FALSE;
			}
			for (j = 0; j < strlen(argv[i]); j++) {
				if (!g_ascii_isalnum(argv[i][j])) {
					printf("password argument contains invalid char\n");
					return FALSE;
				}
			}
			strcpy(cmdlineargs.userpass,argv[i]);
			cmdlineargs.user_set = TRUE;

			continue;
		}
		if (strcmp(argv[i],"--lilo-mbr") == 0) {
/*
  lilo_mode:
  1 - use selected_harddisk MBR
  2 - use selected rootpartition 
  3 - don't use lilo
  4 - use selected bootpartition
*/
			cmdlineargs.lilo_type = 1;
			cmdlineargs.lilo_set = TRUE;
			continue;
		}
		if (strcmp(argv[i],"--lilo-root") == 0) {
			cmdlineargs.lilo_type = 2;
			cmdlineargs.lilo_set = TRUE;
			continue;
		}
		if (strcmp(argv[i],"--nolilo") == 0) {
			cmdlineargs.lilo_type = 3;
			cmdlineargs.lilo_set = TRUE;
			continue;
		}
		if (strcmp(argv[i],"--lilo-boot") == 0) {
			cmdlineargs.lilo_type = 4;
			cmdlineargs.lilo_set = TRUE;
			continue;
		}
		if (strcmp(argv[i],"--ui") == 0) {
			if (++i >= argc)
				return FALSE;
			if (strcmp(argv[i],"cmdline") == 0) {
				cmdlineargs.ui_type = UI_CMDLINE_TYPE;
				cmdlineargs.ui_set = TRUE;
				continue;
			}
			if (strcmp(argv[i],"gtk") == 0) {
				cmdlineargs.ui_type = UI_GTK_TYPE;
				cmdlineargs.ui_set = TRUE;
				continue;
			}	
			if (strcmp(argv[i],"auto") == 0) {
				cmdlineargs.ui_type = UI_AUTO_TYPE;
				cmdlineargs.ui_set = TRUE;
				continue;
			}
		}
		return FALSE;
	}
}
