/* Copyright (C) 2003 Alex de Landgraaf <alextreme@xs4all.nl>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"
#include "installer.h"
#include "instlib.h"
#include "ui.h"

#define _XOPEN_SOURCE
#include <unistd.h>
#include <stdio.h>
#include <dirent.h>

extern GtkWidget *StartScreen;
extern GtkWidget *ErrorMessage;
extern GtkWidget *SelectHarddisk;
extern GtkWidget *SelectSwap;
extern GtkWidget *NoPart;
extern GtkWidget *SelectRoot;
extern GtkWidget *SmallRoot;
extern GtkWidget *NoRoot;
extern GtkWidget *SelectFS;
extern GtkWidget *DoFilesystem;
extern GtkWidget *DoInstall;
extern GtkWidget *EnterHost;
extern GtkWidget *EnterRoot;
extern GtkWidget *EnterUser;
extern GtkWidget *SelectBoot;
extern GtkWidget *EndScreen;
extern GtkWidget *DoSwap;
extern GtkWidget *ErrorLabel;
extern GtkWidget *HelpDialog;
extern GtkWidget *SelectPartExpert;
extern GtkWidget *DataRoot;

/*
void
on_helpbutton_clicked                  (GtkButton       *button,
                                        gpointer         user_data)
{

}*/


void
on_okbuttonStart_clicked               (GtkButton       *button,
                                        gpointer         user_data)
{
	InitInstallprocess();

	SelectHarddisk = create_SelectHarddisk ();
	gtk_widget_show (SelectHarddisk);
	gtk_widget_hide (StartScreen);
}


void
on_okbuttonHarddisk_clicked            (GtkWidget       *widget,
                                        gpointer         user_data)
{	
	GtkWidget *optionmenu = lookup_widget(SelectHarddisk,"optionmenupartitioners");
	debug("selected %s\n",gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(widget)->entry)));

	GList *hdlist = GetHarddiskList(FALSE); /* don't want fuzzy strings */
	GList *hdlist_fluffy = GetHarddiskList(TRUE);
	gint itemnr = GetStringInListNr(hdlist_fluffy,(gchar *)gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(widget)->entry)));
	GList *ptr = GetNthItem(hdlist,itemnr);

	UnmountHarddiskPartitions(ptr->data);
	SaveSelectedHarddisk(ptr->data);
	StartPartitioner(GetSelectedHarddisk(),gtk_option_menu_get_history(GTK_OPTION_MENU(optionmenu)),TRUE);

	FreeStringList(hdlist);
	FreeStringList(hdlist_fluffy);
	SelectSwap = create_SelectSwap ();
	gtk_widget_show (SelectSwap);
	gtk_widget_hide (SelectHarddisk);  
}


void
on_cancelbuttonSwap_clicked            (GtkButton       *button,
                                        gpointer         user_data)
{
	gtk_widget_show(SelectHarddisk);
	gtk_widget_hide(SelectSwap);
}


void
on_okbuttonSwap_clicked                (GtkWidget       *widget,
                                        gpointer         user_data)
{
	// check to see what, if any, swappartition must be used
	debug("selected %s\n",gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(widget)->entry)));
	if (strcmp(gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(widget)->entry)),"No Swap") == 0) {
		UseSwap(FALSE);
	}
	else {
		UseSwap(TRUE);
		GList *fluffy_swap = GetPartList(TRUE,TRUE); /* fluffy swap list */
		GList *no_fluffy_swap = GetPartList(TRUE,FALSE); /* not fluffy */
		int nr = GetStringInListNr(fluffy_swap,(gchar *)gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(widget)->entry)));
		GList *ptr = GetNthItem(no_fluffy_swap,nr);
		SaveSelectedSwap(ptr->data);
		FreeStringList(fluffy_swap);
		FreeStringList(no_fluffy_swap);
	}
	SelectRoot = create_SelectRoot ();
	gtk_widget_show (SelectRoot);
	gtk_widget_hide(SelectSwap);
}

void
on_closebuttonNoPart_clicked           (GtkButton       *button,
                                        gpointer         user_data)
{

}

void
on_cancelbuttonRoot_clicked            (GtkButton       *button,
                                        gpointer         user_data)
{
	gtk_widget_show(SelectSwap);
	gtk_widget_hide(SelectRoot);
}

void
on_buttonOkFS_clicked                  (GtkWidget       *widget,
                                        gpointer         user_data)
{  
}


void
on_buttonOkHost_clicked                (GtkWidget       *widget,
                                        gpointer         user_data)
{
	SetupEtcFiles((gchar *)gtk_entry_get_text(GTK_ENTRY(widget)));
	
	EnterRoot = create_EnterRoot ();
	gtk_widget_show (EnterRoot);
	gtk_widget_hide(EnterHost);
}


void
on_okbuttonRoot_clicked                (GtkWidget       *widget,
                                        gpointer         user_data)
{
	GtkWidget *optionmenu = lookup_widget(SelectRoot,"optionmenuFS2");
	GList *fluffy_root = GetPartList(FALSE,TRUE); /* fluffy rootpart */
	GList *no_fluffy_root = GetPartList(FALSE,FALSE); /* not fluffy, this one is */
	int nr = GetStringInListNr(fluffy_root,(gchar *)gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(widget)->entry)));
	GList *ptr = GetNthItem(no_fluffy_root,nr);
	SaveSelectedRoot(ptr->data);
	FreeStringList(fluffy_root);
	FreeStringList(no_fluffy_root);
	// check to see which filesystem was chosen
	gint sel = gtk_option_menu_get_history(GTK_OPTION_MENU(optionmenu));
	SaveSelectedFilesystemType(sel);
	SetSavedCdromSize(GetCdromSize());
	if (CheckSizeRootpartition()) {
		if (CheckDataRootpartition()) {
			DoInstallProcess();
		}
	}
	// else let DIP be done from SmallRoot callback
}

void
on_buttonCancelNoRoot_clicked          (GtkButton       *button,
                                        gpointer         user_data)
{

}


void
on_cancelFS_clicked                    (GtkButton       *button,
                                        gpointer         user_data)
{
	gtk_widget_show(SelectRoot);
	gtk_widget_hide(SelectFS);
}



void
on_buttonCancelRootPasswd_clicked      (GtkButton       *button,
                                        gpointer         user_data)
{
	gtk_widget_show(EnterHost);
	gtk_widget_hide(EnterRoot);
}



void
on_buttonOkRootPasswd_clicked          (GtkWidget       *widget,
                                        gpointer         user_data)
{
	GtkWidget *entry1 = lookup_widget(widget,"entryRoot1");
	GtkWidget *entry2 = lookup_widget(widget,"entryRoot2");
	gint file;
	gchar cmdline[256];

	// checks for making sure the passwords are valid and reasonably secure
	if (strcmp(gtk_entry_get_text(GTK_ENTRY(entry1)),gtk_entry_get_text(GTK_ENTRY(entry2))) != 0) {
		gtk_entry_set_text(GTK_ENTRY(entry1),"");
		gtk_entry_set_text(GTK_ENTRY(entry2),"");
		ShowErrorBack("Passwords are not identical!\nPlease try again");
		return;
	}
	if (strlen(gtk_entry_get_text(GTK_ENTRY(entry1))) <= 4) {
		gtk_entry_set_text(GTK_ENTRY(entry1),"");
		gtk_entry_set_text(GTK_ENTRY(entry2),"");
		ShowErrorBack("Enter a password of at least 5 characters\nPlease try again");
		return;
	}	
	SetRootPassword((gchar *)gtk_entry_get_text(GTK_ENTRY(entry1)));
	EnterUser = create_EnterUser ();
	gtk_widget_show (EnterUser);
	gtk_widget_hide(EnterRoot);
}


void
on_buttonCancelUser_clicked            (GtkButton       *button,
                                        gpointer         user_data)
{
	gtk_widget_show(EnterRoot);	
	gtk_widget_hide(EnterUser);
}



void
on_buttonOkUser_clicked                (GtkWidget       *widget,
                                        gpointer         user_data)
{	
	GtkWidget *label  = lookup_widget(widget,"labelUserPassError");
	GtkWidget *entry_user = lookup_widget(widget,"entryUser");
	GtkWidget *entry1 = lookup_widget(widget,"entryUserPass1");
	GtkWidget *entry2 = lookup_widget(widget,"entryUserPass2");

	if (strcmp(gtk_entry_get_text(GTK_ENTRY(entry1)),gtk_entry_get_text(GTK_ENTRY(entry2))) != 0) {
		/* popup error */
		gtk_entry_set_text(GTK_ENTRY(entry1),"");
		gtk_entry_set_text(GTK_ENTRY(entry2),"");
		gtk_label_set_text(GTK_LABEL(label),"\nPasswords are not identical!\n\nPlease try again");
		return;
	}
	if (strlen(gtk_entry_get_text(GTK_ENTRY(entry1))) == 0) {
		/* popup error */
		gtk_entry_set_text(GTK_ENTRY(entry1),"");
		gtk_entry_set_text(GTK_ENTRY(entry2),"");
		gtk_label_set_text(GTK_LABEL(label),"\nYou must enter a password!\n\nPlease try again");
		return;
	}


	SetUpHomeDir((gchar *)gtk_entry_get_text(GTK_ENTRY(entry_user)),(gchar *)gtk_entry_get_text(GTK_ENTRY(entry1)));
//	This should be optional and should be realized in postinstall.sh, not in installers binary
//	UseXsessionBoot((gchar *)gtk_entry_get_text(GTK_ENTRY(entry_user)));

	CopyKernel();

	SelectBoot = create_SelectBoot ();
	gtk_widget_show (SelectBoot);
	gtk_widget_hide(EnterUser);
	GtkWidget *combo = lookup_widget(SelectBoot,"LiloHarddiskCombo");
	GtkWidget *boot = lookup_widget(SelectBoot,"radiobuttonLILO4");

	if (boot_selected) {
		gtk_widget_show(boot);
	}
	// if there is more than one harddisk, throw up a combo box so
	// the user can select the right MBR
	GList *hdlist = GetHarddiskList(FALSE);
	if (hdlist != NULL) {
		gtk_combo_set_popdown_strings(GTK_COMBO(combo),hdlist);
		if (hdlist->next != NULL) {
			multiple = TRUE;
			gtk_widget_show(combo);
		}
		FreeStringList(hdlist);
	}
}

void
on_buttonCancelBoot_clicked            (GtkButton       *button,
                                        gpointer         user_data)
{
	gtk_widget_show(EnterUser);
	gtk_widget_hide(SelectBoot);
}

void
on_buttonOkBoot_clicked                (GtkWidget      *widget,
                                        gpointer         user_data)
{
	GtkWidget *lilo = lookup_widget(widget,"radiobuttonLILO");
	GtkWidget *floppy = lookup_widget(widget,"checkbuttonBootFloppy");
	GtkWidget *lilo2 = lookup_widget(widget,"radiobuttonLILO2");
	GtkWidget *lilo3 = lookup_widget(widget,"radiobuttonLILO3");
	GtkWidget *lilo4 = lookup_widget(widget,"radiobuttonLILO4");

	GtkWidget *anim = lookup_widget(widget,"checkbuttonLILOanimate");
	GtkWidget *combo = lookup_widget(widget,"LiloHarddiskCombo");

	GValue *value;

	if (multiple == TRUE) {
		SaveSelectedHarddisk((gchar *)gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(combo)->entry)));
	}

	ExecPreInstShFile();

	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(GTK_CHECK_BUTTON(GTK_RADIO_BUTTON(lilo)))) == TRUE) 
		SetupGrub(1);
	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(GTK_CHECK_BUTTON(GTK_RADIO_BUTTON(lilo2)))) == TRUE) 
		SetupGrub(2);
	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(GTK_CHECK_BUTTON(GTK_RADIO_BUTTON(lilo3)))) == TRUE) 
		SetupGrub(3);
	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(GTK_CHECK_BUTTON(GTK_RADIO_BUTTON(lilo4)))) == TRUE) 
		SetupGrub(4);

	SetupMiscPostInstFiles();

	// execute the mainmodule-dependant postinst.sh script
	// if it exists. You can use this to script around install-related
	// problems
	
	ExecPostInstShFile();
	
	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(GTK_CHECK_BUTTON(floppy))) == TRUE) {
		MakeGrubBootdisk();
	}

	UnmountTarget();

	EndScreen = create_EndScreen ();
	gtk_widget_show (EndScreen);
	gtk_widget_hide(SelectBoot);
}


void
on_buttonOkEnd_clicked                 (GtkButton       *button,
                                        gpointer         user_data)
{

}

void
on_optionmenuHarddisk_realize          (GtkWidget       *widget,
                                        gpointer         user_data)
{


}


void
on_optionmenuSwap_realize              (GtkWidget       *widget,
                                        gpointer         user_data)
{

}


void
on_checkbuttonSwap_toggled             (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{

}


void
on_optionmenuRoot_realize              (GtkWidget       *widget,
                                        gpointer         user_data)
{

}


void
on_optionmenuFS_realize                (GtkWidget       *widget,
                                        gpointer         user_data)
{

}

void
on_optionmenu1_changed                 (GtkOptionMenu   *optionmenu,
                                        gpointer         user_data)
{

}



void
on_SelectHarddisk_show                 (GtkWidget       *widget,
                                        gpointer         user_data)
{
	GtkWidget *next = lookup_widget(SelectHarddisk,"okbuttonHarddisk");
	// add the list of available harddisks to the combo box
	if (cmdlineargs.harddisk_set == TRUE) {
		debug("selected %s\n",cmdlineargs.harddisk);
		SaveSelectedHarddisk(cmdlineargs.harddisk);
		StartPartitioner(GetSelectedHarddisk(),0,TRUE);
		SelectSwap = create_SelectSwap ();
		gtk_widget_show (SelectSwap);
		gtk_widget_hide (SelectHarddisk);
	}
	else {
		GList *hdlist = GetHarddiskList(TRUE);
		if (hdlist == NULL) {
			gtk_widget_set_sensitive(next,FALSE);
		}
		else {
			gtk_widget_set_sensitive(next,TRUE);
		}
		gtk_combo_set_popdown_strings(GTK_COMBO(widget),hdlist);
		FreeStringList(hdlist);
		SetupPartitionerList();
	}
}

void
on_SelectSwap_show                     (GtkWidget       *widget,
                                        gpointer         user_data)
{

	if (cmdlineargs.swappart_set == TRUE ||
	    cmdlineargs.no_swap_partition) {
		debug("selected %s\n",cmdlineargs.rootpart);
		if ((!strcmp(cmdlineargs.rootpart,"No Swap") == 0) &&
		    (!cmdlineargs.no_swap_partition)) {
			UseSwap(FALSE);
		}
		else {
			UseSwap(TRUE);
			SaveSelectedSwap(cmdlineargs.rootpart);
		}
		SelectRoot = create_SelectRoot ();
		gtk_widget_show (SelectRoot);
		gtk_widget_hide(SelectSwap);
	}
	else {
		GList *swaplist = GetPartList(TRUE,TRUE); /* we want a fluffy swappart list !*/
		gtk_combo_set_popdown_strings(GTK_COMBO(widget),swaplist);
		//FreeStringList(swaplist);
	}
}

void
on_SelectRoot_show                     (GtkWidget       *widget,
                                        gpointer         user_data)
{
	if (cmdlineargs.rootpart_set == TRUE &&
	    cmdlineargs.fs_set == TRUE) {
		// for now, you need to use both arguments if you want to skip this one
		SaveSelectedRoot(cmdlineargs.rootpart);
		SaveSelectedFilesystemTypeString(cmdlineargs.fs);
		DoInstallProcess();
		
	}
	else {
		GtkWidget *optionmenu = lookup_widget(SelectRoot,"optionmenuFS2");
		GtkWidget *okbutton = lookup_widget(SelectRoot,"okbuttonRoot");
		gtk_option_menu_set_history(GTK_OPTION_MENU(optionmenu),1);
		GList *list = GetPartList(FALSE,TRUE); /* fluffy rootpart list */
		if (list == NULL) {
			gtk_widget_set_sensitive(okbutton,FALSE);
		}
		else {
			gtk_widget_set_sensitive(okbutton,TRUE);
		}
		gtk_combo_set_popdown_strings(GTK_COMBO(widget),list);
		FreeStringList(list);
	}
}

void
on_ErrorMessage_show                   (GtkWidget       *widget,
                                        gpointer         user_data)
{
	GtkTextBuffer *textbuf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(widget));
	gtk_text_buffer_set_text((textbuf),error_msg,-1);
}

void
on_ErrorLabel_show                     (GtkWidget       *widget,
                                        gpointer         user_data)
{
	GtkWidget *back = lookup_widget(widget,"backbuttonErrorLabel");
	GtkWidget *quit = lookup_widget(widget,"quitbuttonErrorLabel");
	GtkWidget *label = lookup_widget(widget,"labelErrorLabel");
	if (ErrorBackButton()) 
		gtk_widget_hide(quit);
	else
		gtk_widget_hide(back);
	gtk_label_set_text(GTK_LABEL(label),error_msg);
}


void
on_SelectFS_realize                    (GtkWidget       *widget,
                                        gpointer         user_data)
{
}


void
on_DoFilesystem_show                   (GtkWidget       *widget,
                                        gpointer         user_data)
{
	GtkWidget *progbar = lookup_widget(widget,"progressbar10");
	timer = g_timeout_add(1000, (GSourceFunc)ProgressTimeout,progbar);
}


void
on_DoInstall_show                      (GtkWidget       *widget,
                                        gpointer         user_data)
{
	GtkWidget *progbar = lookup_widget(widget,"progressbar11");
	GtkWidget *image = lookup_widget(widget,"imageInstallProcess");
	DoAnimation(image,PACKAGE_DATA_DIR "/" PACKAGE "/pixmaps/dancingpenguin.gif");
	ProgressTimeoutSize(progbar);
	timer = g_timeout_add(10000, (GSourceFunc)ProgressTimeoutSize,progbar);
}


void
on_DoSwap_show                         (GtkWidget       *widget,
                                        gpointer         user_data)
{
	GtkWidget *progbar = lookup_widget(widget,"progressbar18");
	timer = g_timeout_add(1000, (GSourceFunc)ProgressTimeout,progbar);
}


void
on_HelpDialog_show                     (GtkWidget       *widget,
                                        gpointer         user_data)
{
	GtkWidget *image = lookup_widget(HelpDialog,"image27");
	GtkWidget *textview3 = lookup_widget(HelpDialog,"textview3");
	GtkTextIter iter;
	// set up the help screen. Yes, Matty is a teacher at my university
	// and he deserves some hard-earned respect ;) 
	gtk_image_set_from_file(GTK_IMAGE(image),PACKAGE_DATA_DIR "/" PACKAGE "/pixmaps/matty.png");	
	gtk_text_buffer_get_start_iter(GTK_TEXT_BUFFER(gtk_text_view_get_buffer(GTK_TEXT_VIEW(textview3))),&iter);
	gtk_text_view_scroll_to_iter(GTK_TEXT_VIEW(textview3),&iter,0.0,FALSE,0.0,0.0);
}


void
on_textview3_show                      (GtkTextView     *textview,
                                        gchar           *string,
                                        gpointer         user_data)
{

}


void
on_ExpertButtonRoot_clicked            (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkWidget *rootcombo = lookup_widget(SelectRoot,"comboRoot");
	// set root partition prior to starting
	GList *fluffy_list = GetPartList(FALSE,TRUE); /* fluffy rootpart list */
	GList *non_fluffy_list = GetPartList(FALSE,FALSE); /*non-fluffy rootpart list */
	int nr = GetStringInListNr(fluffy_list,(gchar *)gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(rootcombo)->entry))); 
	GList *ptr = GetNthItem(non_fluffy_list,nr);
	SaveSelectedRoot(ptr->data);
	FreeStringList(fluffy_list);
	FreeStringList(non_fluffy_list);
// should create only first time when user press expert partitioning
    if (SelectPartExpert == NULL) {
	SelectPartExpert = create_SelectPartExpert ();

	GtkWidget *expert_selected_wid[MAX_EXPERT_MOUNTS];	
	expert_selected_wid[0] = lookup_widget(SelectPartExpert,"ComboPart1Expert");
	expert_selected_wid[1] = lookup_widget(SelectPartExpert,"ComboPart2Expert");
	expert_selected_wid[2] = lookup_widget(SelectPartExpert,"ComboPart3Expert");
	expert_selected_wid[3] = lookup_widget(SelectPartExpert,"ComboPart4Expert");
	gint j;
	GList * items = GetExpertpartitionList();

	for (j = 0; j < MAX_EXPERT_MOUNTS; j++) {
		gtk_combo_set_popdown_strings(GTK_COMBO(expert_selected_wid[j]),items);	
	}
	FreeStringList(items);
    }
    gtk_widget_show (SelectPartExpert);
}

// TODO: cancel doesn't work as expect - from users view cancel is same like ok
void
on_cancelButtonExpert_clicked          (GtkButton       *button,
                                        gpointer         user_data)
{
	gtk_widget_hide(GTK_WIDGET(SelectPartExpert));
}


void
on_okButtonExpert_clicked              (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkWidget *expert_parts[MAX_EXPERT_MOUNTS];
	GtkWidget *expert_mounts[MAX_EXPERT_MOUNTS];
	GtkWidget *expert_filesys[MAX_EXPERT_MOUNTS];
	GtkWidget *expert_addfstab[MAX_EXPERT_MOUNTS];
	gint i;
	/* still have to make this more purty, but for the time being i couldn't care less */
	expert_parts[0] = lookup_widget(SelectPartExpert,"ComboPart1Expert");
	expert_parts[1] = lookup_widget(SelectPartExpert,"ComboPart2Expert");
	expert_parts[2] = lookup_widget(SelectPartExpert,"ComboPart3Expert");
	expert_parts[3] = lookup_widget(SelectPartExpert,"ComboPart4Expert");
	expert_mounts[0] = lookup_widget(SelectPartExpert,"ComboMount1Expert");
	expert_mounts[1] = lookup_widget(SelectPartExpert,"ComboMount2Expert");
	expert_mounts[2] = lookup_widget(SelectPartExpert,"ComboMount3Expert");
	expert_mounts[3] = lookup_widget(SelectPartExpert,"ComboMount4Expert");
	expert_filesys[0] = lookup_widget(SelectPartExpert,"CheckFilesystem1Expert");
	expert_filesys[1] = lookup_widget(SelectPartExpert,"CheckFilesystem2Expert");
	expert_filesys[2] = lookup_widget(SelectPartExpert,"CheckFilesystem3Expert");
	expert_filesys[3] = lookup_widget(SelectPartExpert,"CheckFilesystem4Expert");
	expert_addfstab[0] = lookup_widget(SelectPartExpert,"CheckFstab1Expert");
	expert_addfstab[1] = lookup_widget(SelectPartExpert,"CheckFstab2Expert");
	expert_addfstab[2] = lookup_widget(SelectPartExpert,"CheckFstab3Expert");
	expert_addfstab[3] = lookup_widget(SelectPartExpert,"CheckFstab4Expert");

	expert_part = TRUE;
	boot_selected = FALSE;

	for (i = 0; i < MAX_EXPERT_MOUNTS; i++) {
		if (strcmp(gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(expert_parts[i])->entry)),"none") == 0) {
			continue;
		}
		if (strcmp(gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(expert_mounts[i])->entry)),"/boot") == 0) {
			SaveSelectedBoot(GTK_ENTRY(GTK_COMBO(expert_parts[i])->entry));
			boot_selected = TRUE;
		}
		strncpy(expert_selected_parts[i],gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(expert_parts[i])->entry)),10);
		strcpy(expert_selected_mounts[i],gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(expert_mounts[i])->entry)));
		expert_new_filesystem[i] = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(GTK_CHECK_BUTTON(expert_filesys[i])));
		expert_fstab_only[i] = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(GTK_CHECK_BUTTON(expert_addfstab[i])));
	}
	gtk_widget_hide(GTK_WIDGET(SelectPartExpert));
}


void
on_SelectRoot_realize                  (GtkWidget       *widget,
                                        gpointer         user_data)
{
	GtkWidget *optionmenu = lookup_widget(SelectRoot,"optionmenuFS2");
	gtk_option_menu_set_history(GTK_OPTION_MENU(optionmenu),1);
}


void
on_buttonBack_clicked                  (GtkButton       *button,
                                        gpointer         user_data)
{
	gtk_widget_hide(SmallRoot);
}


void
on_buttonForward_clicked               (GtkButton       *button,
                                        gpointer         user_data)
{
	gtk_widget_hide(SmallRoot);
	DoInstallProcess(); /* this should be handled more genericaly, though */
}

void
on_backbuttonErrorLabel_clicked        (GtkButton       *button,
                                        gpointer         user_data)
{
	gtk_widget_hide(ErrorLabel);
}


void
on_buttonbackDataRoot_clicked          (GtkButton       *button,
                                        gpointer         user_data)
{
	gtk_widget_hide(DataRoot);
}


void
on_buttonforwardDataRoot_clicked       (GtkButton       *button,
                                        gpointer         user_data)
{
	gtk_widget_hide(DataRoot);
	DoInstallProcess();
}


void
on_combo_entrySelectHarddisk_activate  (GtkEntry        *entry,
                                        gpointer         user_data)
{

}


void
on_combo_entrySelectSwap_activate      (GtkEntry        *entry,
                                        gpointer         user_data)
{

}


void
on_combo_entrySelectRoot_activate      (GtkEntry        *entry,
                                        gpointer         user_data)
{

}


void
on_entryHost_activate                  (GtkEntry        *entry,
                                        gpointer         user_data)
{

}


void
on_entryRootPasswod_activate           (GtkEntry        *entry,
                                        gpointer         user_data)
{

}


void
on_entryUserPass2_activate             (GtkEntry        *entry,
                                        gpointer         user_data)
{

}

