/* Copyright (C) 2003 Alex de Landgraaf <alextreme@xs4all.nl>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include <stdio.h>
#include <stdarg.h>
#include "args.h"
#include "interface.h"
#include "support.h"
#include "installer.h"
#include "cmdline.h"

GtkWidget *StartScreen;
GtkWidget *ErrorMessage;
GtkWidget *SelectHarddisk;
GtkWidget *SelectSwap;
GtkWidget *NoPart;
GtkWidget *SelectRoot;
GtkWidget *SmallRoot;
GtkWidget *NoRoot;
GtkWidget *SelectFS;
GtkWidget *DoFilesystem;
GtkWidget *DoInstall;
GtkWidget *EnterHost;
GtkWidget *EnterRoot;
GtkWidget *EnterUser;
GtkWidget *SelectBoot;
GtkWidget *EndScreen;
GtkWidget *DoSwap;
GtkWidget *ErrorLabel;
GtkWidget *HelpDialog;
GtkWidget *SelectPartExpert;
GtkWidget *DataRoot;

int debug(const char *fmt, ...) {
        int d;
        va_list args;

	va_start(args, fmt);
        if (debug_switch == 1) {
                d = vfprintf(stdout,fmt,args);
                va_end(args);

                return d;
	}
        va_end(args);

	      return 0;
}

gchar *build_cmdline(int argc, char **argv) {
	int i;
	if (argv == NULL || argc < 0) {
		return NULL;
	}
	gchar *cmdline = g_strdup(argv[0]);
	for (i = 1; i < argc; i++) {
		gchar *tmp = g_strdup_printf("%s %s",cmdline,argv[i]);
		g_free(cmdline);
		cmdline = tmp;
	}
	return cmdline;
}

int
main (int argc, char *argv[])
{
	GtkWidget *image;
	gboolean cmdline = FALSE;
#ifdef ENABLE_NLS
	bindtextdomain (GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR);
	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
	textdomain (GETTEXT_PACKAGE);
#endif
	debug_switch = 0;
	gtk_set_locale ();
	if (!check_args(&argc, &argv)) {
		print_help();
		return 1;
	}
	if (cmdlineargs.ui_set == TRUE && cmdlineargs.ui_type == UI_AUTO_TYPE) {
/* Auto works only when root, swap, hdd, nopart, fstype, hostname, rootpass, user, and a lilo option are set. If not, you will need to supply input (for now) */
		if (cmdlineargs.harddisk_set == FALSE ||
		    cmdlineargs.no_partitioning == FALSE ||
		    cmdlineargs.hostname_set == FALSE ||
		    cmdlineargs.rootpass_set == FALSE ||
		    cmdlineargs.user_set == FALSE ||
		    cmdlineargs.lilo_set == FALSE) {
			printf("Auto install doesn't have the necessary commandline arguments\n");
			print_help();
			return 1;
		}
	}
	StoreKernelVersion();
	if (!gtk_init_check (&argc, &argv) || 
	    cmdlineargs.ui_type == UI_CMDLINE_TYPE ||
	    cmdlineargs.ui_type == UI_AUTO_TYPE) {
		// use commandline interface instead
		start_cmdline_iface();
	}
	else {
		add_pixmap_directory (PACKAGE_DATA_DIR "/" PACKAGE "/pixmaps");
		
		HelpDialog = create_HelpDialog();
		
		if (CheckRoot() == TRUE) {
			
			StartScreen = create_StartScreen ();
			gtk_widget_show (StartScreen);
		}
		else {
			//gksu_init(argc,argv);
			//gchar *cmdline = build_cmdline(argc,argv);
			//gksu_run(cmdline);
			//g_free(cmdline);
			ShowError(_("Sorry, you need to be root to use the Gnoppix installer"));
			return 1;
		}
	
		gtk_main ();
	}
	return 0;
}

