/* Copyright (C) 2004 Alex de Landgraaf <alextreme@xs4all.nl>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include <sys/mount.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"
#include "installer.h"
#include "instlib.h"
#include "ui.h"

#define _XOPEN_SOURCE
#include <unistd.h>
#include <stdio.h>
#include <dirent.h>

extern GtkWidget *StartScreen;
extern GtkWidget *ErrorMessage;
extern GtkWidget *SelectHarddisk;
extern GtkWidget *SelectSwap;
extern GtkWidget *NoPart;
extern GtkWidget *SelectRoot;
extern GtkWidget *SmallRoot;
extern GtkWidget *NoRoot;
extern GtkWidget *SelectFS;
extern GtkWidget *DoFilesystem;
extern GtkWidget *DoInstall;
extern GtkWidget *EnterHost;
extern GtkWidget *EnterRoot;
extern GtkWidget *EnterUser;
extern GtkWidget *SelectBoot;
extern GtkWidget *EndScreen;
extern GtkWidget *DoSwap;
extern GtkWidget *ErrorLabel;
extern GtkWidget *HelpDialog;
extern GtkWidget *SelectPartExpert;
extern GtkWidget *DataRoot;

gint global_cdrom_size;
gint global_error_type;

enum {
	ERROR_BACK,
	ERROR_QUIT,
	ERROR_CONTINUE
};

/**
  Timeout function for the progress bars

  @param data the GtkProgressBar to pulse
*/

gboolean ProgressTimeout(gpointer data) {
	GtkWidget *pbar = data;

	if (data == NULL)
		return FALSE;
	gtk_progress_bar_pulse(GTK_PROGRESS_BAR(pbar));
	while (g_main_iteration(FALSE));
	if (timeout == 1) {
		return FALSE;
	}
	return TRUE;
}

void SetSavedCdromSize(gint size) {
	global_cdrom_size = size;
}

gint GetSavedCdromSize(gint size) {
	return global_cdrom_size;
}

/**
   Should be called for every next frame, as GIFs don't have built-in
   frame durations. Increases the animation to the next frame.
   Please note that this function currently only advances the imageInstallProcess
   widget, should be fixed...

   @param data gpointer; pointer to the animation's iterator

   return FALSE if animation should end and timeout has been reached
*/

gboolean DoAnimationFrame(gpointer data) {
	GtkWidget *image = lookup_widget(DoInstall,"imageInstallProcess");
	GdkPixbufAnimationIter *iter = data;

	if (gdk_pixbuf_animation_iter_advance(iter,NULL) == TRUE) {
	  GdkPixbuf *pixbuf = gdk_pixbuf_animation_iter_get_pixbuf(iter);
	  gtk_image_set_from_pixbuf(GTK_IMAGE(image),pixbuf);
	}
	if (timeout == 1)
	  return FALSE;
	return TRUE;
}

/**
   Sets up the image to display the next frame every X millsec.
   
   @param image GtkImage, widget to display the animation
   @param path  path to the animation's file (GIF type)

*/

void DoAnimation(GtkWidget *image, gchar *path) {
	GdkPixbufAnimation *anim = gdk_pixbuf_animation_new_from_file(path,NULL);
	if (anim != NULL) {
	  GdkPixbufAnimationIter *iter = gdk_pixbuf_animation_get_iter(anim,NULL);
	  GdkPixbuf *pixbuf = gdk_pixbuf_animation_iter_get_pixbuf(iter);
      	  gtk_image_set_from_pixbuf(GTK_IMAGE(image),pixbuf);

	  timer = g_timeout_add(100, (GSourceFunc)DoAnimationFrame,iter);
	  //		gtk_image_set_from_animation(GTK_IMAGE(image),anim);
	}
}

/**
   Checks if the size of the currently selected root partition is enough
   Uses globals: global_cdrom_size

   return TRUE if size of the partition is enough
*/

gboolean CheckSizeRootpartition() {
	debug("Info: global cdrom size: %d\nInfo: part used size: %d\n",GetCurrentSize(global_cdrom_size),GetTotalPartSize(GetSelectedRoot()));
	gint expected_size = GetCurrentSize(global_cdrom_size);
	gint available_size = GetTotalPartSize(GetSelectedRoot());
	debug("Info: Expected size: %d Available size on root: %d\n",expected_size,available_size);
	if (expected_size + 100 >= available_size) {
		SmallRoot = create_SmallRoot();
		gtk_widget_show(SmallRoot);
		debug("Warning: Root partition size dangerously low or not enough for safely installing Gnoppix\n");
		return FALSE;
	}
	return TRUE;
}	

/*
	returns if the selected root partition's used size is more than 100
	if so, return TRUE
	if not, or any error, return FALSE
	maybe move the mounting to instlib?
*/

gboolean CheckDataRootpartition() {
	if (!MountPartition(GetSelectedRoot(),TEMP_MOUNT_DIR)) {
		debug("Couldn't check size of root partition\n");
		return FALSE;
	} 
	gint curr_size = GetUsedPartSize(GetSelectedRoot());
	debug("Info: Current size on root: %d\n",curr_size);
	umount(TEMP_MOUNT_DIR);
	if (curr_size >= 100) {
		DataRoot = create_DataRoot();
		gtk_widget_show(DataRoot);
		debug("Info: Root partition already has data, sure to overwrite?\n");
		return FALSE;
	}
	return TRUE;
}	


/**
  Timeout function for the installing-process

  @param data GtkProgressBar widget which should display the occupied size of the current (root)partition

  return FALSE if timeout should be removed
*/

gboolean ProgressTimeoutSize(gpointer data) {
	GtkWidget *pbar = data;
	gchar str_per[5];
	if (data == NULL)
		return FALSE;
	gint curr_rootpart_size = GetUsedPartSize(GetSelectedRoot());
	gdouble fraction = GetCurrentPercentage(global_cdrom_size,curr_rootpart_size);
	debug("Info: Curr rootpart: %d\nInfo: Curr fraction: %f\nInfo: Curr global cdrom size: %d\n",
		curr_rootpart_size,fraction,global_cdrom_size);
	updateInstallLabel(curr_rootpart_size,GetCurrentSize(global_cdrom_size));
	gint percentage = (fraction * 100);
	sprintf(str_per,"%3d%%",percentage);
	gtk_progress_bar_set_text(GTK_PROGRESS_BAR(pbar),str_per);
	gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(pbar),fraction);
	while (g_main_iteration(FALSE));
	if (timeout == 1) {
		return FALSE;
	}
	return TRUE;
}

/**
   Start the copying of files in a separate thread, while displaying
   the user feedback
*/

void DoInstallProcess() {
	GThread *thread;
	if (!g_thread_supported ()) g_thread_init (NULL);

	// make the swap partition
	if(swapon) {
		DoSwap = create_DoSwap ();
		gtk_widget_show (DoSwap);
		gtk_widget_hide(SelectRoot);
		timeout = 0;
		while (g_main_iteration(FALSE));
		swap_done = FALSE;
		thread = g_thread_create((GThreadFunc)MakeSwap,NULL,TRUE,NULL);
		while(swap_done == FALSE) {
			while (g_main_iteration(FALSE));
		}
		g_thread_join(thread);
		timeout = 1;

		// build the filesystem et all
		DoFilesystem = create_DoFilesystem ();
		gtk_widget_show (DoFilesystem);

		gtk_widget_hide(DoSwap);
		timeout = 0;
		while (g_main_iteration(FALSE));
		root_done = FALSE;
		thread = g_thread_create((GThreadFunc)MakeRoot,NULL,TRUE,NULL);
		while(root_done == FALSE) {
			while (g_main_iteration(FALSE));
		}
		g_thread_join(thread);
		timeout = 1;
	}
	else {
		DoFilesystem = create_DoFilesystem ();
		gtk_widget_show (DoFilesystem);
		gtk_widget_hide(SelectRoot);
		timeout = 0;
		while (g_main_iteration(FALSE));
		root_done = FALSE;
		thread = g_thread_create((GThreadFunc)MakeRoot,NULL,TRUE,NULL);
		while(root_done == FALSE) {
			while (g_main_iteration(FALSE));
		}
		g_thread_join(thread);
		
		timeout = 1;
	}
	// install (well, copy) all files
	DoInstall = create_DoInstall ();
	gtk_widget_show (DoInstall);
	gtk_widget_hide(DoFilesystem);
	timeout = 0;
	
	while (g_main_iteration(FALSE));
	install_done = FALSE;
	thread = g_thread_create((GThreadFunc)BuildRoot,NULL,TRUE,NULL);
	
	while(install_done == FALSE) {
		while (g_main_iteration(FALSE));
	}
	// done with install, join our threads and continue with 
	// the configuration process or inform user about failing install
// TODO:
	// if there are 0 free space now installing cannot continue
	
	g_thread_join(thread);
	timeout = 1;

	if (cmdlineargs.hostname_set == TRUE) {
		SetupEtcFiles(cmdlineargs.hostname);
		EnterRoot = create_EnterRoot ();
		gtk_widget_show (EnterRoot);
		gtk_widget_hide(EnterHost);
	}
	else {
		EnterHost = create_EnterHost ();
		gtk_widget_hide(DoInstall);
		gtk_widget_show (EnterHost);
	}
}

gboolean ErrorBackButton() {
	if (global_error_type == ERROR_BACK)
		return TRUE;
	return FALSE;
}

/* for recoverable errors, with the only option of going back */
void ShowErrorBack(gchar *error) {
	error_msg = error;
	ErrorLabel = create_ErrorLabel();
	global_error_type = ERROR_BACK;
	gtk_widget_show(ErrorLabel);
}

/* for non-recoverable errors, only option is quitting */
void ShowError(gchar *error) {
	error_msg = error;
	ErrorLabel = create_ErrorLabel();
	global_error_type = ERROR_QUIT;
	gtk_widget_show (ErrorLabel);
}

void SetupPartitionerList() {
	GtkWidget *optionmenu = lookup_widget(SelectHarddisk,"optionmenupartitioners");
	GtkWidget *menu = gtk_menu_new();
	GtkWidget *item;
	GList *list = GetPartitionerListNames(TRUE);
	GList *ptr = list;
	gtk_option_menu_remove_menu(GTK_OPTION_MENU(optionmenu));
	while(ptr != NULL) {
		item = gtk_menu_item_new_with_label(ptr->data);
		gtk_widget_show(item);
		gtk_menu_shell_append(GTK_MENU_SHELL(menu),item);
		ptr = ptr->next;
	}
	gtk_option_menu_set_menu(GTK_OPTION_MENU(optionmenu),menu);
	FreeStringList(list);
}

void updateInstallLabel(gint current, gint total) {
	GtkWidget *labelInstallprocess = lookup_widget(DoInstall,"labelInstallprocess");
	gchar *label_str = NULL;
	if (current <= 0) {
		label_str = g_strdup_printf("Copying commencing...\n");
	}
	else if (total <= 0) {
		label_str = g_strdup_printf("Couldn't detect size of files in /cdrom,\n%dMiB have been copied\n",current);
	}
	else {
		if (current > total) {
			current = total - 1;
		}
		label_str = g_strdup_printf("%dMiB from a total of approx. %dMiB have been copied\n",current,total);
	}
	gtk_label_set_text(GTK_LABEL(labelInstallprocess),label_str);
	g_free(label_str);
}
