#include "installer.h"
#include "instlib.h"

#include <stdio.h>
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <glib.h>

gchar *ask_harddisk() {
	GList* hdlist = NULL;
	hdlist = (GList *)GetHarddiskList();
	GList* pointer = hdlist;
	gint i = 1;
	gchar inbuf[3];
	char *cp;
	gint choice = 0;
	gboolean success = FALSE;
	if (pointer != NULL && pointer->next == NULL) { // only one harddisk, use it without prompting
		return pointer->data;
	}
	while(!success) {
		printf("Please choose the harddisk you wish to install to:\n");
		while(pointer != NULL) {
			printf("(%d) %s\n",i++,pointer->data);
			pointer = pointer->next;
		}
		fgets(inbuf,3,stdin);
		cp = strrchr(inbuf, '\n');
		if (cp)
			*cp = '\0';
		if (atoi(inbuf) < i) {
			// good input
			choice = atoi(inbuf);
			success = TRUE;
		}
		printf("\n");
	}
	fflush(stdin);
	i = 1;
	pointer = hdlist;
	while (pointer != NULL) {
		if (choice == i) {
			return pointer->data;
		}
		i++;
		pointer = pointer->next;
	}
	// should never get here
	return NULL;
}

gchar *ask_rootpart() {
	GList* rplist = GetPartList(FALSE,FALSE);
	GList* pointer = rplist;
	gint i = 1;
	gchar inbuf[3];
	gint choice = 0;
	char *cp;
	gboolean success = FALSE;
	if (pointer != NULL && pointer->next == NULL) { // only one available partition, using
		return pointer->data;
	}
	while(!success) {
		printf("Please choose the root partition you wish to install to:\n");
		while(pointer != NULL) {
			printf("(%d) %s\n",i++,pointer->data);
			pointer = pointer->next;
		}
		fgets(inbuf,3,stdin);
		cp = strrchr(inbuf, '\n');
		if (cp)
			*cp = '\0';
		if (atoi(inbuf) < i) {
			// good input
			choice = atoi(inbuf);
			success = TRUE;
		}
		printf("\n");
	}
	fflush(stdin);
	i = 1;
	pointer = rplist;
	while (pointer != NULL) {
		if (choice == i) {
			gchar *pdata = g_strdup(pointer->data);
			FreeStringList(rplist);
			return pdata;
		}
		i++;
		pointer = pointer->next;
	}
	// should never get here
	return NULL;
}


gchar *ask_swappart() {
	GList* splist = GetPartList(TRUE,FALSE);
	GList* pointer = splist;
	gint i = 1;
	gchar inbuf[3];
	gint choice = 0;
	char *cp;
	gboolean success = FALSE;
	if (cmdlineargs.ui_type == UI_AUTO_TYPE) {
		if (pointer != NULL && strcmp(pointer->next->data,"No Swap") == 0) { // assuming to use the only available swap partition
			return pointer->data;
		}
	}
	while(!success) {
		printf("Please choose the swap partition you wish to use:\n");
		while(pointer != NULL) {
			printf("(%d) %s\n",i++,pointer->data);
			pointer = pointer->next;
		}
		fgets(inbuf,3,stdin);
		cp = strrchr(inbuf, '\n');
		if (cp)
			*cp = '\0';
		if (atoi(inbuf) < i) {
			// good input
			choice = atoi(inbuf);
			success = TRUE;
		}
		printf("\n");
	}
	fflush(stdin);
	i = 1;
	pointer = splist;
	while (pointer != NULL) {
		if (choice == i) {
			gchar *pdata = g_strdup(pointer->data);
			FreeStringList(splist);
			return pdata;
		}
		i++;
		pointer = pointer->next;
	}
	// should never get here
	return NULL;
}

gchar *ask_fstype() {
	gint i = 0;
	gchar inbuf[3];
	gint choice = 0;
	gboolean success = FALSE;
	char *cp;
	while(!success) {
		printf("Please choose the filesystem type you would like to use (1 and 2 are save bets):\n");

		printf("(1) ext2\n(2) ext3\n(3) reiserfs\n(4) xfs\n");
		fgets(inbuf,3,stdin);
		cp = strrchr(inbuf, '\n');
		if (cp)
			*cp = '\0';
		if (atoi(inbuf) >= 1 && atoi(inbuf) <= 4) {
			// good input
			choice = atoi(inbuf);
			success = TRUE;
		}
		printf("\n");
	}
	fflush(stdin);
	switch(choice) {
	case(1):
		return "ext2";
	case(2):
		return "ext3";
	case(3):
		return "reiserfs";
	case(4):
		return "xfs";
	}
	// shouldn't get here
	return NULL;
}

gchar *ask_hostname() {
	gchar hostname[64];
	gboolean success = FALSE;
	char *cp;
	while(!success) {
		printf("Please enter the hostname of your fresh install:\n");

		fgets(hostname,63,stdin);
		cp = strrchr(hostname, '\n');
		if (cp)
			*cp = '\0';
		gint j;
		for (j = 0; j < strlen(hostname); j++) {
			if (!g_ascii_isalnum(hostname[j])) {
				printf("hostname contains invalid char\n");
				continue;
			}
		}
		success = TRUE;
		printf("\n");
	}
	fflush(stdin);
	gchar *returnhn;
	returnhn = (gchar *)malloc(64);
	strcpy(returnhn,hostname); // needs fixing
	return returnhn;
}

gchar *read_pass() {
	gchar *cp = NULL;
	gchar *pass = (gchar *)malloc(64);
	fgets(pass,63,stdin);
	cp = strrchr(pass, '\n');
	if (cp)
		*cp = '\0';
	fflush(stdin);
	return pass;
}

gchar *ask_rootpass() {
	gchar *rootpass;
	gboolean success = FALSE;
	static FILE *ifp;

	while(!success) {
		//	printf("Please enter a password for the root user, nothing will be shown while you enter it:\n");
		printf("Please enter a password for the root user:\n");
		rootpass = read_pass();
		if (rootpass == NULL) {
			printf("Password reading error occurred, please try again\n");
		}
		else 
			success = TRUE;
		printf("\n");
	}
	return rootpass;
}
gchar * ask_username() {
	gchar username[64];
	gboolean success = FALSE;
	char *cp;
	while(!success) {
		printf("Please enter the username of your regular user account:\n");

		fgets(username,63,stdin);
		cp = strrchr(username, '\n');
		if (cp)
			*cp = '\0';
		gint j;
		for (j = 0; j < strlen(username); j++) {
			if (!g_ascii_isalnum(username[j])) {
				printf("username contains invalid char\n");
				continue;
			}
		}
		success = TRUE;
		printf("\n");
	}
	fflush(stdin);
	gchar *returnun;
	returnun = (gchar *)malloc(64);
	strcpy(returnun,username); // needs fixing
	return returnun;
}
gchar * ask_userpass() {
	gchar *userpass;
	gboolean success = FALSE;
	static FILE *ifp;

	while(!success) {
//		printf("Please enter a password for the user, nothing will be shown while you enter it:\n");
		printf("Please enter a password for the user:\n");
		userpass = read_pass();
		if (userpass == NULL) {
			printf("Password reading error occurred, please try again\n");
		}
		else 
			success = TRUE;
		printf("\n");
	}
	return userpass;
}

gint ask_lilo_type() {
	gchar inbuf[3];
	gint choice = 0;
	gboolean success = FALSE;
	char *cp;
	while(!success) {
		printf("Please choose a bootloader option:\n");

		printf("(1) Install LILO to mbr\n(2) Install LILO to the root partition\n(3) Don't install LILO\n");
		fgets(inbuf,2,stdin);
		cp = strrchr(inbuf, '\n');
		if (cp)
			*cp = '\0';
		if (atoi(inbuf) >= 1 && atoi(inbuf) <= 3) {
			// good input
			choice = atoi(inbuf);
			success = TRUE;
		}
		printf("\n");
	}
	fflush(stdin);
	return choice;
}
void show_swap_message() {
	printf("Making swap partition...\n");
}
void show_root_message() {
	printf("Making root partition...\n");
}
	
void show_copy_message() {
	printf("Copying all files to new partition, this will take a while\nGet some coffee, walk the dog or something. We'll have a progress bar in here sometime once someone bothers making one...\n");
}
void show_kernel_message() {
	printf("Copying and installing the kernel...\n");
}
void show_end_message() {
	printf("Your Gnoppix system is ready! Reboot, take out the install medium and happy morphing!\n");
}
void reboot_system() {
	printf("Rebooting system after auto install is disabled, but Gnoppix has been installed succesfully!\nReboot, take out the install medium and happy morphing!\n");
}


void start_cmdline_iface() {
	printf("Welcome to the Gnoppix Installer, commandline version\nPlease note that this installer is still under construction,\nAlex de Landgraaf and gnoppix.org take no responsibility\nfor any damage done.\nThis installation will copy Gnoppix onto your harddisk and optionally set up a bootloader. Your mileage may vary.\n");
	InitInstallprocess();

	/* harddisk section */

	if (cmdlineargs.harddisk_set == FALSE) {
		strcpy(cmdlineargs.harddisk,ask_harddisk());
	}
	printf("Using harddisk: %s\n",cmdlineargs.harddisk);
	SaveSelectedHarddisk(cmdlineargs.harddisk);

	/* partitioning section */

	if (!cmdlineargs.no_partitioning) {
		if (cmdlineargs.partitioner_set == TRUE) {
			StartAltPartitioner(GetSelectedHarddisk(),cmdlineargs.partitioner,FALSE);
		}
		else {
			if (cmdlineargs.ui_type == UI_AUTO_TYPE) { // if auto, don't partition...
			}
			else { 
				printf("Partitioning has been disabled, please exit and use 'cfdisk' if you wish to partition your harddisk...\n");
				//StartPartitioner(selected_harddisk,0,FALSE);
			}
		}
	}

	/* root/swap partition section */

	if (cmdlineargs.swappart_set == FALSE &&
	    cmdlineargs.no_swap_partition == FALSE) {
		strcpy(cmdlineargs.swappart,ask_swappart());
	}
	if (cmdlineargs.no_swap_partition ||
	    strcmp(cmdlineargs.swappart,"No Swap") == 0) {
		UseSwap(FALSE);
	}
	else {
		UseSwap(TRUE);
		SaveSelectedSwap(cmdlineargs.swappart);
	}
	if (cmdlineargs.rootpart_set == FALSE) {
		strcpy(cmdlineargs.rootpart,ask_rootpart());
	}
	SaveSelectedRoot(cmdlineargs.rootpart);

	/* fs section */

	if (!cmdlineargs.fs_set) {
		if (cmdlineargs.ui_type == UI_AUTO_TYPE) {
			strcpy(cmdlineargs.fs,"ext3"); //auto-use ext3
		}
		else {
			strcpy(cmdlineargs.fs,ask_fstype());
		}
	}
	SaveSelectedFilesystemTypeString(cmdlineargs.fs);

	/* copy/install section */

	show_swap_message();
	MakeSwap(NULL);
	show_root_message();
	MakeRoot(NULL);
	show_copy_message();
	BuildRoot(NULL);
	show_kernel_message();
	CopyKernel();

	/* hostname section */
	
	if (!cmdlineargs.hostname_set) {
		if (cmdlineargs.ui_type == UI_AUTO_TYPE) {
			strcpy(cmdlineargs.hostname,"Gnoppix");
		}
		else {
			strcpy(cmdlineargs.hostname,ask_hostname()); // returns unfreeable!
		}
	}
	SetupEtcFiles(cmdlineargs.hostname);

	/* rootpass/user section */

	if (!cmdlineargs.rootpass_set) {
		if (cmdlineargs.ui_type == UI_AUTO_TYPE) {
			strcpy(cmdlineargs.rootpass,"morphix");
		}
		else {
			strcpy(cmdlineargs.rootpass,ask_rootpass());
		}
	}
	SetRootPassword(cmdlineargs.rootpass);

	if (!cmdlineargs.user_set) {
		if (cmdlineargs.ui_type == UI_AUTO_TYPE) {
			strcpy(cmdlineargs.username,"morph");
			strcpy(cmdlineargs.username,"morph");
		}
		else {
			strcpy(cmdlineargs.username,ask_username()); // returns unfreeable!
			strcpy(cmdlineargs.userpass,ask_userpass());
		}
	}
	SetUpHomeDir(cmdlineargs.username,cmdlineargs.userpass);
//UseXsessionBoot should be optional and realized in postinstall.sh, not in installers binar
//	UseXsessionBoot(cmdlineargs.username);

	/* lilo section */

	if (!cmdlineargs.lilo_set) {
		if (cmdlineargs.ui_type == UI_AUTO_TYPE) {
			cmdlineargs.lilo_type = 1; // auto-use mbr
		}
		else {
			cmdlineargs.lilo_type = ask_lilo_type();
		}
	}
	SetupGrub(cmdlineargs.lilo_type);

	SetupMiscPostInstFiles();
	if(cmdlineargs.ui_type == UI_AUTO_TYPE) {
		reboot_system();
		return;
	}
	show_end_message();
}

