#ifndef INSTLIB_H
#define INSTLIB_H

#include <glib.h>

/* I18n/gettext part */
#ifdef ENABLE_NLS
#  include <libintl.h>
#  undef _
#  define _(String) dgettext (PACKAGE, String)
#  ifdef gettext_noop
#    define N_(String) gettext_noop (String)
#  else
#    define N_(String) (String)
#  endif
#else
#  define textdomain(String) (String)
#  define gettext(String) (String)
#  define dgettext(Domain,Message) (Message)
#  define dcgettext(Domain,Message,Type) (Message)
#  define bindtextdomain(Domain,Directory) (Domain)
#  define _(String) (String)
#  define N_(String) (String)
#endif
/* */

#define REAL_INSTALL
#define MAX_EXPERT_MOUNTS 4
#define MAX_MINIMODULES 64

#define PATH_MNT_DIR "/mnt"
#define PIVOT_ROOT_PATH "/usr/sbin/pivotroot"
#define GRUB_MENU_LOCATION "/boot/grub/menu.lst"
#define PROC_CMDLINE "/proc/cmdline"
#define PROC_PARTITIONS "/proc/partitions"
#define PROC_MODULES "/proc/modules"

#define CFDISK_PATH "/sbin/cfdisk"
#define PM_PATH "/usr/sbin/partitionmorpher"
#define QTPARTED_PATH "/usr/sbin/qtparted"
#define PARTED_PATH "/sbin/parted"

/* these should be cleaned up or something 
   sorry if you thought this was well-written, shoo shoo */

gchar *error_msg;
gint timeout;
gint timer;

gchar expert_selected_parts[4][40];
gchar expert_selected_mounts[4][40];
gboolean expert_new_filesystem[4];
gboolean expert_fstab_only[4];
gboolean expert_part;
gboolean boot_selected;
gchar *initrd;
gboolean root_done;
gboolean swap_done;
gboolean install_done;
gboolean swapon;
gboolean multiple;

gboolean cpio_passthrough;

gchar * GetSelectedHarddisk();
gchar * GetSelectedSwap();
gchar * GetSelectedRoot();

/*
  sets up a number of variables 
  should be used first
*/

void InitInstallprocess();

gchar *GetSelectedFilesystemName();
void FreeInstallProcess();
gboolean MountPartition(gchar *partition, gchar *destination);

/*
  copy the version of the kernel into the kernel_version array
*/

void StoreKernelVersion();

/*
  A nice small function that just executes a command.
  Does block the current GUI, but most of the time this isn't a problem.
  Can also use it in a different thread, like when copying all the files over
  to the rootpartition.
  Note that normally the returntype should be checked, but as not every
  execution is necessarily fatal it's not done often in here
*/

gboolean ExecuteCommand(gchar *command);

/*
  Stores the current kernel version in kernel_version
*/

void StoreKernelVersion();

/*
  Saves 'harddisk' in 'selected_harddisk'
*/

void SaveSelectedHarddisk(gchar *harddisk);

/*
  Saves 'swappart' in 'selected_swappart'
*/

void SaveSelectedSwap(gchar *swappart);

/*
  Saves 'rootpart' in 'selected_rootpart'
*/

void SaveSelectedRoot(gchar *rootpart);

/*
  Saves the name of the filesystem type to 'selected_fs'
  0 - ext2
  1 - ext3
  2 - reiserfs
  3 - xfs
  anything else - ext3
*/

void SaveSelectedFilesystemType(gint selected);
void SaveSelectedFilesystemTypeString(gchar *filesystem);
/*
  starts cfdisk using 'harddisk', might be extended in the future
  X - cfdisk in a terminal
  Alt - define partition line
  XAlt - define partition line, executed in terminal
*/

void StartPartitioner(gchar *harddisk, gint partnr, gboolean xfree86);
void StartAltPartitioner(gchar *harddisk,gchar *partitioner, gboolean xfree86);


/*
  sets if swap should be used
*/

void UseSwap(gboolean use_swap);

/*
  attempts to set up the swappartition
  'data' is given so this function can be gthreaded
*/

gpointer MakeSwap(gpointer data);

/*
  attempts to set up the rootpartition
  'data' is given so this function can be gthreaded
*/

gpointer MakeRoot(gpointer data);

/*
  copies all files to the rootpartition
  'data' is given so this function can be gthreaded
*/

gpointer BuildRoot(gpointer data);

/*
  copies the kernel to the root/boot partition
*/

void CopyKernel();

/*
  used for setting up a few files that need to know the new hostname
*/

void SetupEtcFiles(gchar *hostname);

/*
  used for setting the root password. not checked for length, make it <30
*/

void SetRootPassword(gchar *password);

/*
  sets up the group and user, homedir, and sets the password.
  make sure both are less than 30 to be sure FIXME
*/

void SetUpHomeDir(gchar *username, gchar *password);

/*
  lilo_mode/grub_mode
  1 - use selected_harddisk MBR
  2 - use selected rootpartition 
  3 - don't use lilo/grub
  4 - use selected bootpartition
*/

gboolean SetupGrub(gint grub_mode);
gboolean SetupLilo(gint lilo_mode);

/*
  do this after setting up lilo
*/

void SetupMiscPostInstFiles();

/*
  checks for /morphix/gtkinstaller/preinst.sh
  use this before lilo
*/

void ExecPreInstShFile();

/*
  checks for /morphix/gtkinstaller/postinst.sh
  use this before lilo
*/

void ExecPostInstShFile();

/*
  Attempts to make a bootdisk
  FIXME - this currently doesn't work right
*/

void MakeGrubBootdisk();
void MakeBootdisk();

/*
  Attempts to unmount the target rootpartition
  FIXME - this currently doesn't work right
*/

void UnmountTarget();

/*
  gets a GList of available harddisk partitions
*/

GList* GetHarddiskList();
GList *GetNthItem(GList *list,gint nr);
gint GetStringInListNr(GList *list, gchar *str);
void FreeStringList(GList *list);
GList *GetPartitionerListCommands(gchar *harddisk_str,gboolean xfree86);
GList *GetPartitionerListNames(gboolean xfree86);

/*
  gets a GList of available swappartitions
  NOTE: this currently also reads in the list of rootpartitions,
  use GetRootpartList to retrieve it
*/

GList *GetPartList(gboolean swap,gboolean fluffy);

/*
  checks to see if the current user is root. if not, give an error and return FALSE
*/

gboolean CheckRoot();

/*
  get a list of available extra partitions usable for the extra partition option
*/

GList *GetExpertpartitionList();

/*
  returns the total size of the cdrom used in MB's
  returns -1 if the cdrom couldn't be found
*/

gint GetCdromSize();

/*
  returns the used/total size of the partition in MB's
  returns -1 if the partition couldn't be found
*/
gint GetTotalPartSize(gchar *partition);
gint GetUsedPartSize(gchar *partition);

/*
  returns the current size of the to be installed system
  in effect, it just multiplies the value with 2.8,
  as that's about the compression ratio with cloop.
  it's messy, but this can always be improved :)
*/

gint GetCurrentSize(gint cdrom_size);

/*
  needs the size of the cdrom (installer media), and the size of the root partition (including any extra partitions).
  will return the percentage copied to the root partition
*/

gdouble GetCurrentPercentage(gint cdrom_size, gint curr_rootpart_size);
#endif
